@@ -87,8 +87,6 @@ except Exception:
 def test_pickle_dump_load(assertion, source, target=None,
         protocol=(0, HIGHEST_PROTOCOL)):
     start, stop = protocol
-    if start > HIGHEST_PROTOCOL:
-        return
     failures = []
     for protocol in range(start, stop+1):
         try:
@@ -105,8 +103,6 @@ def test_pickle_dump_load(assertion, source, target=None,
 def test_pickle_exception(assertion, exception, obj,
         protocol=(0, HIGHEST_PROTOCOL)):
     start, stop = protocol
-    if start > HIGHEST_PROTOCOL:
-        return
     failures = []
     for protocol in range(start, stop+1):
         try:
@@ -602,7 +598,7 @@ class TestEnum(unittest.TestCase):
             test_pickle_exception(
                     self.assertRaises, PicklingError, self.NestedEnum.twigs,
                     protocol=(0, 3))
-            test_pickle_dump_load(self.assertIs, self.NestedEnum.twigs,
+            test_pickle_dump_load(self.assertTrue, self.NestedEnum.twigs,
                     protocol=(4, HIGHEST_PROTOCOL))
 
     def test_exploding_pickle(self):
@@ -1161,7 +1157,7 @@ class TestEnum(unittest.TestCase):
             self.assertEqual(NI5, 5)
             test_pickle_dump_load(self.assertEqual, NI5, 5, protocol=(4, 4))
             self.assertEqual(NEI.y.value, 2)
-            test_pickle_dump_load(self.assertIs, NEI.y, protocol=(4, 4))
+            test_pickle_dump_load(self.assertTrue, NEI.y, protocol=(4, 4))
 
     def test_subclasses_with_reduce(self):
         class NamedInt(int):
